#include <fcntl.h>
#include <io.h>
#include "bytitura.h"

// compile line:
// cl DemoDosHeader.cpp /link /OUT:"DemoDosHeader.btt32" "kernel32.lib" "BttDemo32.lib" /DLL /SUBSYSTEM:WINDOWS

// 3 necessarily functions:
// ApiVersionUsed()
// IKnowThisFile()
// ParseFile()

/////////////////////////////////////////////////////////////////////////////

 long __stdcall  DllMain( void *hModule, unsigned long reason, void *lpReserved ){ return 1; }

/////////////////////////////////////////////////////////////////////////////

typedef struct{
    unsigned short   e_magic;    // Magic number
    unsigned short   e_cblp;     // Bytes on last page of file
    unsigned short   e_cp;       // Pages in file
    unsigned short   e_crlc;     // Relocations
    unsigned short   e_cparhdr;  // Size of header in paragraphs
    unsigned short   e_minalloc; // Minimum extra paragraphs needed
    unsigned short   e_maxalloc; // Maximum extra paragraphs needed
    unsigned short   e_ss;       // Initial SS value
    unsigned short   e_sp;       // Initial SP value
    unsigned short   e_csum;     // Checksum
    unsigned short   e_ip;       // Initial IP value
    unsigned short   e_cs;       // Initial CS value
    unsigned short   e_lfarlc;   // File address of relocation table
}DOS_HEADER;

/////////////////////////////////////////////////////////////////////////////

 bool  ReadHeader( const wchar_t *pfilename, long *pfilesize, DOS_HEADER *p_dos_header )
{
 int  h =  ::_wopen( pfilename, _O_RDONLY | _O_BINARY );
 if(  h == -1 )
   return false;

 if( ::_read( h, p_dos_header, sizeof(DOS_HEADER) ) != sizeof(DOS_HEADER) )
   p_dos_header->e_magic = 0;

 *pfilesize = ::_filelength( h );

 ::_close( h );

 return p_dos_header->e_magic == 0x5A4D 
     || p_dos_header->e_magic == 0x4D5A;
}

/////////////////////////////////////////////////////////////////////////////
// 1st necessarily function
 BTT_EXPORT int  ApiVersionUsed()
{
 return BTT_API_VERSION;
}

/////////////////////////////////////////////////////////////////////////////
// 2nd necessarily function
 BTT_EXPORT bool  IKnowThisFile( BTT_PREFILE_DATA *p_btt_prefile_data )
{
 long        filesize;
 DOS_HEADER  dos_header;
 if( ReadHeader( p_btt_prefile_data->_filename, &filesize, &dos_header ) == false )
   return false;                                                               // not my file

 p_btt_prefile_data->_file_flags    = file_dialog_data_little_endian           // only LE
                                    | file_dialog_data_enable                  // file may be as data
                                    | file_dialog_code16_enable;               // file may be as code 16bits 
 p_btt_prefile_data->_base_address  = 0;                                       // base address
 p_btt_prefile_data->_codesize      = 0;                                       // first: file as data
 p_btt_prefile_data->_decode_engine = decode_engine_ia32;                      // disassembler for code, decode_engine_none if no code 

 return true;                                                                  // yes, i'm know
}

/////////////////////////////////////////////////////////////////////////////
// 3rd necessarily function
 BTT_EXPORT bool  ParseFile( BTT_FILE_DATA *p_btt_file_data )
{
 long        filesize;
 DOS_HEADER  dos_header;
 ReadHeader( p_btt_file_data->_filename, &filesize, &dos_header );

 const int file_index = p_btt_file_data->_file_index; 

 BttFileLog( file_index, L"DemoDosHeader is running..." );                     // line to log pane

//
//  Only one segment in DEMO
//
 long  segment_size = dos_header.e_cp * 512/*filepage*/ + dos_header.e_cblp;
 if(   segment_size > filesize )
   segment_size = filesize;
 if( p_btt_file_data->_codesize == 0 )                                         // user select file as data..
   BttFileAddDataSegment(  file_index, 0, 0, segment_size, 0 );                // ..define data segment
  else                                                                         // or
   BttFileAddCodeSegment(  file_index, 0, 0, segment_size, 0 );                // ..define code segment
 BttFileSegmentName(       file_index, 0, L"DEMO" );                           // lets name for segment

//
//  Header via sture file
// 
 BttFileMakeStruct( file_index, 0, L"DOS_HEADER" );
 
//
// Relocations
//
 if( dos_header.e_crlc ){
   const int offset_segment = 1+1/*word + word*/;
   BttFileCommentline(  file_index, dos_header.e_lfarlc, L"Relocations (offset:segment)" );                          // comment line at address
   BttFileMakePerline(  file_index, offset_segment );                                                                // numbers per line
   BttFileMakeWords(    file_index, dos_header.e_lfarlc, dos_header.e_crlc * offset_segment );                       // list of WORDs
   BttFileMakeCollapse( file_index, dos_header.e_lfarlc, dos_header.e_crlc * offset_segment * sizeof(short), true ); // define region of bytes and collapse it
 }

//
// Body
//
 long  body_offset = dos_header.e_cparhdr * 16/*paragraph*/;
 long  body_size   = segment_size - body_offset;
 BttFileCommentline( file_index, body_offset, L"Body" );                       // comment line at address
 if( p_btt_file_data->_codesize == 0 )                                         // user select file as data..
   BttFileMakeDump( file_index, body_offset, body_size, false );               // ..show as dump
  else                                                                         // or
   BttFileMakeCode( file_index, body_offset );                                 // ..show as code

 return true;                                                                  // parsing is success
}

////////////////////////////////////////////////////////////
